local LrErrors = import 'LrErrors'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrPathUtils = import "LrPathUtils"

local function apiUrl(path)
  local base_url = "https://studiocms.io"

  return base_url .. path
end

local api = {}

function api.connect(pt)
  pt.orgs = {}
  pt.orgDetails = {}
  pt.contributor = false

  LrFunctionContext.postAsyncTaskWithContext("ConnectTask", function(context)
    LrDialogs.attachErrorDialogToFunctionContext(context)

    pt.connected = false
    pt.connecting = true

    local success, result, headers = LrFunctionContext.pcallWithContext(
      'HitAuthenticateEndpoint',
      function(context)
        local headers = { { field = "Content-Type", value = "application/json" } }

        local url = apiUrl("/api/auth/login")

        local params = Json.encode({ email = pt.accountEmail, password = pt.password })

        return LrHttp.post(url, params .. '\r\n', headers)
      end
    )

    -- Report failure or connect
    pt.connecting = false
    if success and headers.status == 200 then
      local data = Json.decode(result)
      pt.connected = true
      pt.auth_token = data.token
    elseif headers.status == 401 then
      LrErrors.throwUserError('Incorrect email or password')
    else
      LrErrors.throwUserError('Something went wrong. Please try again.')
    end
  end)
end

function api.loadGalleries(pt)
  pt.galleries = {}
  pt.gallery_titles = {}

  if (pt.auth_token == nil or pt.auth_token == '') then
    return {}
  end

  local success, result, headers = LrFunctionContext.pcallWithContext(
    'listGalleries',
    function(context)
      local url = apiUrl("/api/galleries")
      local token = pt.auth_token
      local headers = { { field = "Authorization", value = "Bearer " .. token } }

      return LrHttp.get(url, headers)
    end
  )

  if (success) then
    local json = Json.decode(result)
    local galleries = json.data
    local gallery_titles = {}

    for _, gallery in pairs(galleries) do
      Logger.outputToLog(Logger.table_to_string(gallery))
      table.insert(gallery_titles, { title = gallery.title, value = gallery.id })
    end

    pt.galleries = galleries
    pt.gallery_titles = gallery_titles

    Logger.outputToLog(Logger.table_to_string(pt.gallery_titles))
  end
end

function api.upload(path, contentType, token, gallery_id)
  local f, errMsg = io.open(path, "rb")
  local file_data
  local filename = LrPathUtils.leafName(path)

  if f then
    file_data = f:read("*a")
    f:close()
  else
    LrErrors.throwUserError(errMsg)
  end

  local uploadBody
  local postHeaders
  local success, result, headers = LrFunctionContext.pcallWithContext(
    'UploadPhoto',
    function(context)
      local delimiter = "--PART_DELIMITER"

      uploadBody = '--' .. delimiter .. "\r\n"
          .. 'Content-Disposition: form-data; name="gallery_id"\r\n'
          .. '\r\n'
          .. gallery_id .. '\r\n'
          .. '--' .. delimiter .. '\r\n'
          .. 'Content-Disposition: form-data; name="' .. filename .. '"; filename="' .. filename .. '"\r\n'
          .. 'Content-Type: ' .. contentType .. '\r\n'
          .. '\r\n'
          .. file_data .. '\r\n'
          .. '--' .. delimiter .. "--" .. '\r\n'

      postHeaders = { { field = "Authorization", value = "Bearer " .. token } }

      postHeaders[#postHeaders + 1] = { field = "Content-Type", value = "multipart/form-data; boundary=" .. delimiter }

      return LrHttp.post(
        apiUrl("/api/photos/multipart"),
        uploadBody,
        postHeaders,
        "POST",
        60 * 30
      )
    end
  )

  if not success then
    LrErrors.throwUserError("Failed to upload to StudioCMS")
  elseif not headers then
    LrErrors.throwUserError("No HTTP headers returned from StudioCMS")
  end

  local json_result = Json.decode(result)
  local data = json_result.data[1]

  return data.id
end

return api
